* Stata program for the paper "A Stochastic Frontier Model with an
* Endogenous Treatment Status and a Mediator". For evaluation only.
* Hung-Jen Wang, wangh@ntu.edu.tw

myclear
set more off
set trace off
set matsize 2000

capture log close
* log using 0pub_pgm1_log2.txt, text replace

use data4, clear

****** setup ****

set seed 1256

global xvar  lfertvalue lgca predam frann lelevage pc1 pc2
global xvarB lfertvalueB lgcaB predamB frannB lelevageB pc1B pc2B
global zvar  rain2
global zvarB rain2B
global Z_predictors dss_riverkm dss_km2 dss_elev1 dss_elev2 dss_elev3 dss_elev4 dss_sdistrict1 dss_sdistrict2 dss_sdistrict3 dss_sdistrict4 dss_sdistrict5 dss_sdistrict6 dss_km2_upstream /*
 */ dss_riverkm_upstream dss_elev1_upstream dss_elev2_upstream dss_elev3_upstream dss_elev4_upstream dss_sdistrict1_upstream dss_sdistrict2_upstream dss_sdistrict3_upstream dss_sdistrict4_upstream dss_sdistrict5_upstream dss_sdistrict6_upstream noup /*
 */ pleadamsum_code61 pleadamsum_code61_upstream _Iyear_1957 _Iyear_1958 _Iyear_1959 _Iyear_1960 _Iyear_1961 _Iyear_1962 _Iyear_1963 _Iyear_1964 _Iyear_1965 _Iyear_1966 _Iyear_1967 _Iyear_1968 _Iyear_1969 _Iyear_1970 _Iyear_1971 _Iyear_1972 _Iyear_1973 _Iyear_1974 _Iyear_1975 _Iyear_1976 _Iyear_1977 _Iyear_1978 _Iyear_1979 _Iyear_1980 _Iyear_1981 _Iyear_1982 _Iyear_1983 _Iyear_1984 _Iyear_1985 _Iyear_1986 _Iyear_1987 _Iyear_1988 _Iyear_1989 _Iyear_1990 _Iyear_1991 _Iyear_1992 _Iyear_1993 _Iyear_1994 _Iyear_1995 _Iyear_1996 _Iyear_1997 _Iyear_1998 _Iyear_1999 _Iyear_2000 _Iyear_2004 _Istatenam_2 _Istatenam_3 _Istatenam_4 _Istatenam_5 _Istatenam_6 _Istatenam_7 _Istatenam_8 _Istatenam_9 _Istatenam_10 _Istatenam_11 _Istatenam_12 _Istatenam_13



**********************
* ***** Z_ind *****
**********************


*******************************************
***** constructing the weights ***********
******************************************

   *** principal components ***


pca  $Z_predictors, components(2)

global Z_predictors_pca
forvalues i = 1/2 {
   capture  drop pc`i'  /* drop the dummies that were created earlier */
   global Z_predictors_pca $Z_predictors_pca pc`i'  /* append the names */
}

predict double $Z_predictors_pca

 /* adjust Z2_ind so that it satisfies the model's assumptions */

capture drop e
reg Z2_inst Z1_inst $xvar $zvar $Z_predictors_pca

predict double e, resid
replace Z2_inst = e


****** The Q_{z1} function **************
****** Get the alpha_z1 parameter of the paper *******

  quie probit Z1_inst $xvar $zvar  $Z_predictors_pca

  quie predict double Q_z1

  local b_low = 0.1
  local b_up  = 1-0.1

  quie replace Q_z1 = cond(Q_z1<0.1, 0.1, cond(Q_z1>0.9, 0.9, Q_z1))


*********** The f_{z_2} function, a mixture normal *************
***********   Get the alpha_f parameters of the paper   *************

  gen byte m1 = 1
  gen byte m2 = 1

    ml model lf mixnormal (Z2_inst = m1, nocons) (Z2_inst= m2, nocons)  (p1: ) (s1: ) (s2: )
    ml init   -1.368926  1.109845  -.2098054  -.0035635  .0067127, copy
    ml max, diff
    mat mixb0 = e(b)

       scalar _m1 = [eq1]_b[m1]
       scalar _m2 = [eq2]_b[m2]
       scalar _pb1  = exp([p1]_b[_cons])/(1 + exp([p1]_b[_cons])) /* the probability */
       scalar _sig1 = exp([s1]_b[_cons])
       scalar _sig2 = exp([s2]_b[_cons])

*********** the M and the D models *************
******** Estimating Q_M and Q_D jointly; P(M,D|..) in the paper *********

quie sum Mvar
global nofobs = r(N)

biprobit (eq1: Mvar = Dvar Z2_inst   $xvar  $zvar  $Z_predictors_pca  ) (eq2: Dvar = Z1_inst   $xvar  $zvar   $Z_predictors_pca  ), nrtol(1e-5) gtol(1e-5)

mat bp0 = e(b)
mat bp0Mz2= bp0[1,"eq1:Z2_inst"]
  scalar alpha_z = bp0Mz2[1,1]

mat bp0Dz1 = bp0[1, "eq2:Z1_inst"]
  scalar gamma_z = bp0Dz1[1,1]
  if scalar(gamma_z) < 0 {
    di in red "Gamma_z cannot be <0."
    aaaaa
  }

mat bp0Deq = bp0[1, "eq2:"]
mat bp0Dx = bp0Deq[1, 2...]

mat bp0Meq = bp0[1, "eq1:"]
mat bp0Mx = bp0Meq[1, 3...]

mat bp0Md = bp0[1,"eq1:Dvar"]
  scalar alpha_d = bp0Md[1,1]

mat bp0rho = bp0[1, "/athrho"]

scalar rho_e = e(rho)

  ***************** Constructing the Weights *******************************************
  ************* the f functions are the density functions of mixture normal ************
  **************************************************************************************

  quie gen double lambda = (Z1_inst - Q_z1)/(Q_z1*(1-Q_z1))
  quie gen double f_z2_b = scalar(_pb1)*normalden(Z2_inst, scalar(_m1), scalar(_sig1)) + /*
      */               (1-scalar(_pb1))*normalden(Z2_inst, scalar(_m2), scalar(_sig2))
  quie gen double f_z2_a = scalar(_pb1)*normalden(Z2_inst + scalar(alpha_d)/scalar(alpha_z), scalar(_m1), scalar(_sig1)) + /*
      */               (1-scalar(_pb1))*normalden(Z2_inst + scalar(alpha_d)/scalar(alpha_z), scalar(_m2), scalar(_sig2))
  quie gen double f_z2_c = scalar(_pb1)*normalden(Z2_inst - scalar(alpha_d)/scalar(alpha_z), scalar(_m1), scalar(_sig1)) + /*
      */               (1-scalar(_pb1))*normalden(Z2_inst - scalar(alpha_d)/scalar(alpha_z), scalar(_m2), scalar(_sig2))
  quie gen double phi1 = f_z2_a/f_z2_b
  quie gen double phi2 = f_z2_c/f_z2_b

           /* smooth the probability */
       quie sum phi2 if phi2 <=15
       quie replace phi2 = r(max) if phi2 > 15  /* 1 of them has extremely large values, akin to outliers */

  quie gen double W11 = Dvar*lambda
  quie gen double W10 = Dvar*phi1*lambda
  quie gen double W01 = (Dvar-1)*phi2*lambda
  quie gen double W00 = (Dvar-1)*lambda



   ***** Calculating the potential mediator function E[M(d')|X,C] = m_{d'}(X, alpha_m) ******


   *** First, computes elements of Psi(Z2,X,eta) ******

predict double tem1, eq(eq2) xb
gen double Xgammax = tem1 - scalar(gamma_z)*Z1_inst

predict double tem2, eq(eq1) xb
gen double Xalphax = tem2 - scalar(alpha_d)*Dvar - scalar(alpha_z)*Z2_inst

drop tem1 tem2

gen double Psi_comp1 = 1/(normal(scalar(gamma_z) + Xgammax) - normal(Xgammax))

quie gen double double_int_d0 = .
quie gen double double_int_d1 = .


 *** Second, do the double integration ****

    /* this box is for examiniation */
    local hasdata = 0
    sort code61

    /* ************************** */

quie sum code61, meanonly
local nofobs = r(N)



  forvalues i = 1/`nofobs' {
     scalar bb1 = Xalphax[`i']
     scalar mylb = Xgammax[`i']
     scalar myub = scalar(gamma_z) + scalar(mylb)
     scalar zeroc = 0
       /* for m_{d'=1} */
     quie myint5a, alphadd(alpha_d) alphaz(alpha_z) xalphax(bb1) rho(rho_e) mylow(mylb) myup(myub) sigma1(_sig1) sigma2(_sig2) m1(_m1) m2(_m2) prob(_pb1) saveas(my_res)
            quie replace double_int_d1 = my_res if _n == `i'
       /* for m_{d'=0} */
     quie myint5a, alphadd(zeroc) alphaz(alpha_z) xalphax(bb1) rho(rho_e) mylow(mylb) myup(myub) sigma1(_sig1) sigma2(_sig2) m1(_m1) m2(_m2) prob(_pb1) saveas(my_res)
            quie replace double_int_d0 = my_res if _n == `i'
  }


     ***** now, adds up ******

quie gen double m_d0 = Psi_comp1*double_int_d0 /* for m_{d'=0}  */
quie gen double m_d1 = Psi_comp1*double_int_d1 /* for m_{d'=1}  */

sort code61


****** Start Estimation

keep code61 lprodmain $xvar $zvar  Dvar Mvar Z1_inst Z2_inst $Z_predictors $Z_predictors_pca W11 W10 W01 W00 year m_d0 m_d1

quie gen myid = _n /* for sorting purpose */

capture drop mysample
capture drop wght

capture quie replace frann2 = frann2*10


mat b0_gmm1 = 0
mat b0_gmm2 = 0

capture drop hasmiss
findmis lprodmain $xvar $zvar W11 W10 W01 W00 Dvar m_d1 m_d0, gen(hasmiss)


foreach X in lprodmain $xvar $zvar {
 di " "
 di "`X'"
 ttest `X' if hasmiss == 0, by(Dvar) unpaired
}


*** initial values, from previous model ****

mat myini1 = (   .6513029,   .0488417,   .9800765,   1.155915,    /*
   */  -.0312174,   -.0686682,   .114844,   -.3355233,   .217235,    /*
   */  1.687552,   .2658189,   -.1110996,   -.6561467,   -.0020588,    /*
   */  .1689316,   -.6845366,   4.39796,   -3.116878,   2.613611,    /*
   */  -2.737709)


  scalar getout = 0

  forvalues i = 1/2 { /* run twice, the 2nd time using the first time's result as init. So that the result is the same as the bootstrap's. */

     if getout == 1 {
        exit
     }

     if `i' == 2 {
        mat myini1 = b0_gmm1
     }


       gmm sftreat_GMM4 if Dvar == 1, nequations(20) parameters($xvar b0 $xvarB b0B $zvar g0 $zvarB g0B)  ///
           mylhs(lprodmain) myxvar($xvar) myzvar($zvar) mywght(W11) mywghtB(W10) ///
           myp(m_d1) mypB(m_d0) ///
           winitial(identity) ///
           from(myini1)     ///
           onestep  ///
           conv_vtol(1e-5)  conv_nrtol(1e-3)   technique(dfp)

       **** further checking ***********

          local missingv = 0 /* flag */
          if _rc == 0 {
            mat checkv = e(V) /* V-COV matrix */
            local vsize = colsof(checkv)
            forvalues i = 1/`vsize' {
              local checkstd = checkv[`i', `i']
              if `checkstd' == 0 | `checkstd' > 2e+7 {
                 local missingv = 1
              }
            }
          }

       ************************

  if _rc == 0 & `missingv' == 0 {
    mat b0_gmm1 = e(b)
  }
  else {
    scalar getout = 1
    exit
  }

   if getout ==1 {
    exit
      }

}


  if getout ==1 {
   exit
   }


***************


  *** obtain H_11, G_11 etc., for LATE ****


    /* begin with the H_11 */

  foreach X in hb_11 hb_10 H_11 {
    capture drop `X'
  }

 quie gen double hb_11 = 0 if hasmiss ==0
 quie gen double hb_10 = 0 if hasmiss ==0

 foreach X in $xvar b0 {
   mat aa_11 = b0_gmm1[1, "`X':_cons"]
   scalar aa_11 = aa_11[1,1]
   mat aa_10 = b0_gmm1[1, "`X'B:_cons"]
   scalar aa_10 = aa_10[1,1]

   if "`X'" ~= "b0" {
     quie replace hb_11 = hb_11 + scalar(aa_11)*`X' if hasmiss ==0
     quie replace hb_10 = hb_10 + scalar(aa_10)*`X' if hasmiss ==0
   }
   else {
     quie replace hb_11 = hb_11 + scalar(aa_11) if hasmiss ==0
     quie replace hb_10 = hb_10 + scalar(aa_10) if hasmiss ==0
   }
 }
    quie gen double H_11 = (m_d1)*hb_11 + (1-m_d1)*hb_10 if hasmiss ==0


    /* Now G_11 */

  foreach X in gb_11 gb_10 G_11 {
    capture drop `X'
  }

 quie gen double gb_11 = 0 if hasmiss ==0
 quie gen double gb_10 = 0 if hasmiss ==0

 foreach X in $zvar g0 {
   mat aa_11 = b0_gmm1[1, "`X':_cons"]
   scalar aa_11 = aa_11[1,1]
   mat aa_10 = b0_gmm1[1, "`X'B:_cons"]
   scalar aa_10 = aa_10[1,1]

   if "`X'" ~= "g0" {
     quie replace gb_11 = gb_11 + scalar(aa_11)*`X' if hasmiss ==0
     quie replace gb_10 = gb_10 + scalar(aa_10)*`X' if hasmiss ==0
   }
   else {
     quie replace gb_11 = gb_11 + scalar(aa_11) if hasmiss ==0
     quie replace gb_10 = gb_10 + scalar(aa_10) if hasmiss ==0
   }
 }
    quie gen double G_11 = (m_d1)*exp(gb_11) + (1-m_d1)*exp(gb_10) if hasmiss ==0


  *************

if getout ==1 {
 exit
}

**** using results from previous models as initial values ****

mat myini2 = (   .7817897,   .2031261,   -.2854763,   1.051983,    /*
   */  -.6062741,   .0196417,   -.049328,   -.5279208,   .4213905,    /*
   */  .0519451,   -2.744842,   -1.532993,   .6190847,   -.0002647,    /*
   */  .0040251,   -.7882092,   3.160318,   -1.473108,   2.569029,    /*
   */  -2.521874)

  scalar getout = 0

  forvalues i = 1/2 { /* run twice, the 2nd time using the first time's result as init. So that the result is the same as the bootstrap's. */

    if getout == 1 {
       exit
    }

     if `i' == 2 {
        mat myini2 = b0_gmm2
     }

       gmm sftreat_GMM4 if Dvar == 0, nequations(20) parameters($xvar b0 $xvarB b0B $zvar g0 $zvarB g0B)  ///
           mylhs(lprodmain) myxvar($xvar) myzvar($zvar) mywght(W01) mywghtB(W00) ///
           myp(m_d1) mypB(m_d0) ///
           winitial(identity) ///
           from(myini2)   ///
           /* tracelevel(gradient) */ onestep  conv_vtol(1e-5)  conv_nrtol(1e-3)  technique(dfp)

  **** further checking ***********

          local missingv = 0 /* flag */
          if _rc == 0 {
            mat checkv = e(V) /* V-COV matrix */
            local vsize = colsof(checkv)
            forvalues i = 1/`vsize' {
              local checkstd = checkv[`i', `i']
              if `checkstd' == 0 | `checkstd' > 2e+7 {
                 local missingv = 1
              }
            }
          }

 ************************************

  if _rc == 0 & `missingv' == 0 {
    mat b0_gmm2 = e(b)
  }
  else {

    scalar getout = 1

    exit
  }
    if getout == 1 {
      exit
    }
}

  if getout == 1 {
    exit
  }


  *** obtain H_01, H_00, G_01, G_00, etc., for LATE ****

  /* begin with H01, H00 */

  foreach X in hb_01 hb_00 H_01 H_00 {
    capture drop `X'
  }

 quie gen double hb_01 = 0 if hasmiss ==0
 quie gen double hb_00 = 0 if hasmiss ==0


 foreach X in $xvar b0 {
   mat aa_01 = b0_gmm2[1, "`X':_cons"]
   scalar aa_01 = aa_01[1,1]
   mat aa_00 = b0_gmm2[1, "`X'B:_cons"]
   scalar aa_00 = aa_00[1,1]

   if "`X'" ~= "b0" {
     quie replace hb_01 = hb_01 + scalar(aa_01)*`X' if hasmiss ==0
     quie replace hb_00 = hb_00 + scalar(aa_00)*`X' if hasmiss ==0
   }
   else {
     quie replace hb_01 = hb_01 + scalar(aa_01) if hasmiss ==0
     quie replace hb_00 = hb_00 + scalar(aa_00) if hasmiss ==0
   }
 }

    quie gen double H_01 = (m_d1)*hb_01 + (1-m_d1)*hb_00 if hasmiss ==0
    quie gen double H_00 = (m_d0)*hb_01 + (1-m_d0)*hb_00 if hasmiss ==0


  /* Now do G_01, G_00 */

  foreach X in gb_01 gb_00 G_01 G_00 {
    capture drop `X'
  }

 quie gen double gb_01 = 0 if hasmiss ==0
 quie gen double gb_00 = 0 if hasmiss ==0


 foreach X in $zvar g0 {
   mat aa_01 = b0_gmm2[1, "`X':_cons"]
   scalar aa_01 = aa_01[1,1]
   mat aa_00 = b0_gmm2[1, "`X'B:_cons"]
   scalar aa_00 = aa_00[1,1]

   if "`X'" ~= "g0" {
     quie replace gb_01 = gb_01 + scalar(aa_01)*`X' if hasmiss ==0
     quie replace gb_00 = gb_00 + scalar(aa_00)*`X' if hasmiss ==0
   }
   else {
     quie replace gb_01 = gb_01 + scalar(aa_01) if hasmiss ==0
     quie replace gb_00 = gb_00 + scalar(aa_00) if hasmiss ==0
   }
 }

    quie gen double G_01 = (m_d1)*exp(gb_01) + (1-m_d1)*exp(gb_00) if hasmiss ==0
    quie gen double G_00 = (m_d0)*exp(gb_01) + (1-m_d0)*exp(gb_00) if hasmiss ==0


  *************


   *** calculate LATE (see Identification of CLATE and LATE in the paper)

     probit Dvar Z1_inst $xvar $zvar  $Z_predictors_pca  if hasmiss == 0, iterate(50)

     foreach X in linear linear0 linear1 Delta Delta_i tem1 tem2 tem3 {
      capture drop `X'
     }
     quie predict double linear  if e(sample), xb
     quie gen double linear0 = linear - _b[Z1_inst]*Z1_inst + _b[Z1_inst] if e(sample)
     quie gen double linear1 = linear - _b[Z1_inst]*Z1_inst  if e(sample)
     quie gen double Delta_i = normal(linear0) - normal(linear1) if e(sample)
     quie sum Delta_i if e(sample)
     scalar Delta = r(mean)

     foreach X in 11 01 00 {
        capture drop tem1
        quie gen double tem1 = Delta_i*(H_`X')/(scalar(Delta))
        quie sum tem1
        scalar H_`X'_hat = r(mean)

        capture drop tem1
        quie gen double tem1 = Delta_i*(G_`X')/(scalar(Delta))
        quie sum tem1
        scalar G_`X'_hat = r(mean)
     }

      scalar LATE = scalar(H_11_hat) - scalar(G_11_hat) -scalar(H_00_hat) + scalar(G_00_hat)
      scalar DLATE = scalar(H_11_hat) - scalar(G_11_hat) -scalar(H_01_hat) + scalar(G_01_hat)
      scalar ILATE = scalar(H_01_hat) - scalar(G_01_hat) -scalar(H_00_hat) + scalar(G_00_hat)

      scalar LATE  = scalar(H_11_hat) - scalar(G_11_hat) -scalar(H_00_hat) + scalar(G_00_hat)
      scalar LATEh = scalar(H_11_hat)  -scalar(H_00_hat)
      scalar LATEg = -(- scalar(G_11_hat)  + scalar(G_00_hat))

      scalar DLATEh = scalar(H_11_hat)  -scalar(H_01_hat)
      scalar DLATEg =  -(- scalar(G_11_hat) + scalar(G_01_hat))
      scalar ILATEh = scalar(H_01_hat)  -scalar(H_00_hat)
      scalar ILATEg =  -(- scalar(G_01_hat)  + scalar(G_00_hat))


di LATE
di DLATE
di ILATE

di LATEh
di DLATEh
di ILATEh

di LATEg
di DLATEg
di ILATEg
